#include "color_picker.h"

#ifndef MIN
#define MIN(a,b) (((a)<(b)) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a,b) (((a)>(b)) ? (a) : (b))
#endif

int fs = 0;
Fl_Double_Window* mainwindow;

void quit() {
  exit(0);
}

void toggle_fullscreen() {
  if (fs) {
        box->resize(5, 5, 790, 275);
        fs = 0;
  } else {
        box->resize(0, 0, 800, 480);
        fs = 1;
  }
  mainwindow->redraw();
}

void change_color(Fl_Color c) {
  box->color(c);
  box->redraw();
  float r = ((c & 0xFF000000) >> 24)/256.0;
  float g = ((c & 0x00FF0000) >> 16)/256.0;
  float b = ((c & 0x0000FF00) >> 8)/256.0;
  float max = MAX(MAX(r,g),b);
  float min = MIN(MIN(r,g),b);

  hue->value(1.0 - max);
  float delta = max - min;

  if (delta > 0.0001)
    {
      saturation->value(delta / max);
      float h;
      if (r == max)
        {
          h = (g - b) / delta;
          if (h < 0.0)
            h += 6.0;
        }
      else if (g == max)
        {
          h = 2.0 + (b - r) / delta;
        }
      else if (b == max)
        {
          h = 4.0 + (r - g) / delta;
        }

      h /= 6.0;
      hue->value(h);
    }
  else
    {
      saturation->value(0.0);
      hue->value(0.0);
    }
}

void set_color() {

  float h = hue->value();
  float s = saturation->value();
  float v = 1.0 - value->value();
  float r,g,b;

  // borrowed from the Gimp
  if (s == 0.0)
    {
      r = v;
      g = v;
      b = v;
    }
  else
    {
      float hue = h;

      if (hue == 1.0) hue = 0.0;

      hue *= 6.0;

      int i = (int) hue;
      float f = hue - i;
      float w = v * (1.0 - s);
      float q = v * (1.0 - (s * f));
      float t = v * (1.0 - (s * (1.0 - f)));

      switch (i)
        {
        case 0:
          r = v;
          g = t;
          b = w;
          break;
        case 1:
          r = q;
          g = v;
          b = w;
          break;
        case 2:
          r = w;
          g = v;
          b = t;
          break;
        case 3:
          r = w;
          g = q;
          b = v;
          break;
        case 4:
          r = t;
          g = w;
          b = v;
          break;
        case 5:
          r = v;
          g = w;
          b = q;
          break;
        }
    }
    box->color(fl_rgb_color(255*r,255*g,255*b));
    box->redraw();
}

#include "color_picker.cxx"

int main() {
        mainwindow = make_window();
        mainwindow->fullscreen();
        mainwindow->show();
        return Fl::run();
}

